@@ -1,7 +1,8 @@
-// Copyright (c) 2023-2024 Sophie Herold
+// Copyright (c) 2023-2025 Sophie Herold
 // Copyright (c) 2023 Alice Mikhaylenko
 // Copyright (c) 2023 Christopher Davis
 // Copyright (c) 2023 Lubosz Sarnecki
+// Copyright (c) 2024 Fina Wilke
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,13 +24,13 @@
 //! This widget it similar to `AdwCarousel`.
 
 use std::cell::{Cell, OnceCell, RefCell};
+use std::sync::LazyLock;
 
 use adw::prelude::*;
 use adw::subclass::prelude::*;
 use glib::Properties;
 use indexmap::IndexMap;
 use log::error;
-use once_cell::sync::Lazy;
 
 use crate::deps::*;
 use crate::widgets::LpImagePage;
@@ -141,8 +142,8 @@ mod imp {
         }
 
         fn signals() -> &'static [Signal] {
-            static SIGNALS: Lazy<Vec<Signal>> =
-                Lazy::new(|| vec![Signal::builder("target-page-reached").build()]);
+            static SIGNALS: LazyLock<Vec<Signal>> =
+                LazyLock::new(|| vec![Signal::builder("target-page-reached").build()]);
             SIGNALS.as_ref()
         }
 
@@ -159,20 +160,32 @@ mod imp {
                 .upper_overshoot(true)
                 .build();
 
-            swipe_tracker.connect_begin_swipe(glib::clone!(@weak obj => move |_| {
-                obj.scroll_animation().pause();
-                obj.step_animation().pause();
-            }));
-
-            swipe_tracker.connect_update_swipe(glib::clone!(@weak obj => move |_, position| {
-                obj.set_position(position);
-            }));
-
-            swipe_tracker.connect_end_swipe(glib::clone!(@weak obj => move |_, velocity, to| {
-                if let Some(page) = obj.page_at(to) {
-                    obj.scroll_to(&page, velocity);
+            swipe_tracker.connect_begin_swipe(glib::clone!(
+                #[weak]
+                obj,
+                move |_| {
+                    obj.scroll_animation().pause();
+                    obj.step_animation().pause();
                 }
-            }));
+            ));
+
+            swipe_tracker.connect_update_swipe(glib::clone!(
+                #[weak]
+                obj,
+                move |_, position| {
+                    obj.set_position(position);
+                }
+            ));
+
+            swipe_tracker.connect_end_swipe(glib::clone!(
+                #[weak]
+                obj,
+                move |_, velocity, to| {
+                    if let Some(page) = obj.page_at(to) {
+                        obj.scroll_to(&page, velocity);
+                    }
+                }
+            ));
 
             self.swipe_tracker.set(swipe_tracker).unwrap();
 
@@ -180,8 +193,12 @@ mod imp {
             let scroll_controller =
                 gtk::EventControllerScroll::new(gtk::EventControllerScrollFlags::HORIZONTAL);
 
-            scroll_controller.connect_scroll(
-                glib::clone!(@weak obj => @default-return glib::Propagation::Proceed, move |_, x, _| {
+            scroll_controller.connect_scroll(glib::clone!(
+                #[weak]
+                obj,
+                #[upgrade_or]
+                glib::Propagation::Proceed,
+                move |_, x, _| {
                     let direction_sign = if obj.imp().is_rtl() { -1. } else { 1. };
 
                     if x * direction_sign > 0. {
@@ -207,8 +224,8 @@ mod imp {
                             glib::Propagation::Stop
                         }
                     }
-                }),
-            );
+                }
+            ));
 
             obj.add_controller(scroll_controller);
         }
@@ -339,6 +356,9 @@ mod imp {
             let position_shift = self.position_shift.get() as f32;
             let page_position = page_index as f32;
 
+            // Reset from potential stacked cards animation
+            page.set_opacity(1.);
+
             // reverse page order for RTL languages
             let direction_sign = if self.is_rtl() { -1. } else { 1. };
 
@@ -489,7 +509,7 @@ impl LpSlidingView {
 
         if let Some(index) = self.index_of(page) {
             // Only unparent if far enough away to not be involved in delete animation
-            if current_index.map_or(false, |x| x.abs_diff(index) as isize > 1) {
+            if current_index.is_some_and(|x| x.abs_diff(index) as isize > 1) {
                 page.unparent();
             }
 
@@ -578,7 +598,7 @@ impl LpSlidingView {
         if let Some(index) = self.index_of(page) {
             if self
                 .current_index()
-                .map_or(false, |x| (x as i64 - index as i64).abs() == 1)
+                .is_some_and(|x| (x as i64 - index as i64).abs() == 1)
             {
                 self.step_to(page);
             } else {
@@ -600,6 +620,9 @@ impl LpSlidingView {
 
                 *self.imp().position_tracking.borrow_mut() =
                     PositionTracking::StackedCards(StackedCards::new(prev_page, backward));
+                // Set current page here as well, to make sure it's set when animation completes
+                // for disabled animations
+                self.set_current_page(Some(page));
                 animation.play();
             }
         }
@@ -614,8 +637,8 @@ impl LpSlidingView {
             animation.set_value_from(self.position());
             animation.set_value_to(index as f64 - self.position_shift());
             animation.set_initial_velocity(initial_velocity);
-            animation.play();
             self.set_current_page(Some(page));
+            animation.play();
         } else {
             log::error!("Page not in LpSlidingView {}", page.file().uri());
         }
@@ -712,9 +735,11 @@ impl LpSlidingView {
 
     fn scroll_animation(&self) -> &adw::SpringAnimation {
         self.imp().scroll_animation.get_or_init(|| {
-            let target = adw::CallbackAnimationTarget::new(
-                glib::clone!(@weak self as obj => move |position| obj.set_position(position)),
-            );
+            let target = adw::CallbackAnimationTarget::new(glib::clone!(
+                #[weak(rename_to = obj)]
+                self,
+                move |position| obj.set_position(position)
+            ));
 
             let animation = adw::SpringAnimation::builder()
                 .spring_params(&adw::SpringParams::new(
@@ -726,9 +751,11 @@ impl LpSlidingView {
                 .target(&target)
                 .build();
 
-            animation.connect_done(glib::clone!(@weak self as obj => move |_| {
-                obj.emit_target_page_reached()
-            }));
+            animation.connect_done(glib::clone!(
+                #[weak(rename_to = obj)]
+                self,
+                move |_| obj.emit_target_page_reached()
+            ));
 
             animation
         })
@@ -736,12 +763,18 @@ impl LpSlidingView {
 
     fn step_animation(&self) -> &adw::TimedAnimation {
         self.imp().step_animation.get_or_init(|| {
-            let target: adw::CallbackAnimationTarget = adw::CallbackAnimationTarget::new(
-                glib::clone!(@weak self as obj => move |progress| {
-                    obj.imp().position_tracking.borrow_mut().set_progress(progress);
-                    obj.queue_allocate();
-                }),
-            );
+            let target: adw::CallbackAnimationTarget =
+                adw::CallbackAnimationTarget::new(glib::clone!(
+                    #[weak(rename_to = obj)]
+                    self,
+                    move |progress| {
+                        obj.imp()
+                            .position_tracking
+                            .borrow_mut()
+                            .set_progress(progress);
+                        obj.queue_allocate();
+                    }
+                ));
 
             let animation = adw::TimedAnimation::builder()
                 .duration(STEP_DURATION)
@@ -751,17 +784,23 @@ impl LpSlidingView {
                 .target(&target)
                 .build();
 
-            animation.connect_done(glib::clone!(@weak self as obj => move |_| {
-                if let Some(current_index) = obj.current_index() {
-                    let imp = obj.imp();
-                    imp.position_tracking.borrow_mut().set_progress(1.);
-                    imp.position_tracking.borrow_mut().set_position(current_index as f64 - obj.position_shift());
-                    obj.queue_allocate();
-                } else {
-                    error!("No current page at end of animation");
+            animation.connect_done(glib::clone!(
+                #[weak(rename_to = obj)]
+                self,
+                move |_| {
+                    if let Some(current_index) = obj.current_index() {
+                        let imp = obj.imp();
+                        imp.position_tracking.borrow_mut().set_progress(1.);
+                        imp.position_tracking
+                            .borrow_mut()
+                            .set_position(current_index as f64 - obj.position_shift());
+                        obj.queue_allocate();
+                    } else {
+                        error!("No current page at end of animation");
+                    }
+                    obj.emit_target_page_reached();
                 }
-                obj.emit_target_page_reached();
-            }));
+            ));
 
             animation
         })
